//DoublePendul.java

import nl.kun.Engine3D.*;
import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import java.net.*;
import java.io.*;
import java.applet.*;
import java.util.*;
/**
 * The main applet class. It draws and controls all buttons, panels and textfields. 
 * Besides that it communicates with the PhysCalcs class, the class that does all the 
 * physics.
 */
public class DoublePendul extends Applet implements TimerCallback, ActionListener
{	
	/** Actionlistener that keeps track of the activity of the buttons and textboxes*/
	private ActionListener theListner;
	/** Textfield of the mass of the pendula */
	private TextField 	textFMass;
	/** Textfield of the Length of the pendula */
	private TextField 	textFLength;
	/** Textfield of the initial angle of pendul1 */
	private TextField	textFalpha0;
	/** Textfield of the initial angle of pendul2 */
	private TextField 	textFbeta0;
	/** Textfield of spring constant */
	private TextField 	textFK;
	/** Textfield of the distance between suspension points of the pendula and the spring */
	private TextField 	textFC;
	/** Most northern panel */
	private Panel 		TopPanel 	= new Panel();
	/** Most eastern panel */
	private Panel 		RightPanel 	= new Panel();
	/** Panel for the textfields */
	private Panel 		TextFieldPanel 	= new Panel();
	/** Most southern panel */
	private Panel 		BottomPanel 	= new Panel();
	/** Most western panel  */
	private Panel 		LeftPanel 	= new Panel();
	/** Button for play/pause */
	private Button 		PlayPauseButton;
	/** Reset button*/
	private Button 		ResetButton;
	/** String for the play/pause button */
        private String 		Splay		= "Play/Pause";
        /** String for the reset button*/
        private String 		Sreset		= "Reset";
        /** Height of the Applet in pixels */
        private int 		HeightApplet;
        /** Height of the Applet in pixels */
        private int 		WidthApplet;
        /** Timer based on the TimerCallback interface */
        private TickTimer	timer;
        /** Class that does the physical calculations */
        private TPhysCalcs 	PhysCalcs;
  /**
   * Routine that is called by the JVM after the construction of the Applet and before the screen is drawn.
   */         
  public void init()
  {   	// Most code in this part speaks for itself.
  	this.setBackground(Color.white);
  	this.setLayout(new BorderLayout()); 
  	timer 		= new TickTimer(this,50);
  	PhysCalcs	= new TPhysCalcs();     
  	textFMass	= new TextField(4);
  	textFLength	= new TextField(4);
  	textFalpha0	= new TextField(4);
  	textFbeta0	= new TextField(4);
  	textFK		= new TextField(4);
  	textFC		= new TextField(4);
  	textFMass.setBackground(Color.white);
  	textFLength.setBackground(Color.white);
  	textFalpha0.setBackground(Color.white);
  	textFbeta0.setBackground(Color.white);
  	textFK.setBackground(Color.white);
	textFC.setBackground(Color.white);
	textFMass.addActionListener( this);
	textFLength.addActionListener( this);
	textFalpha0.addActionListener( this);
	textFbeta0.addActionListener( this);
	textFK.addActionListener( this);
	textFC.addActionListener( this);
	GridLayout LayoutGrid43 = new GridLayout(4,3,2,2); 
	TextFieldPanel.setLayout(LayoutGrid43);
	TextFieldPanel.add(new Label("Mass(kg)"));
	TextFieldPanel.add(new Label("Length(m)"));
	TextFieldPanel.add(new Label(new String("\u03B1")+"_0(deg)"));
	TextFieldPanel.add(textFMass);
	TextFieldPanel.add(textFLength);
	TextFieldPanel.add(textFalpha0);
        TextFieldPanel.add(new Label("k(N/m)"));
        TextFieldPanel.add(new Label("C(m)"));
        TextFieldPanel.add(new Label(new String("\u03B2")+"_0(deg)"));
        TextFieldPanel.add(textFK);
        TextFieldPanel.add(textFC);
        TextFieldPanel.add(textFbeta0);
        PlayPauseButton	= new Button(Splay);
        ResetButton	= new Button(Sreset);
        PlayPauseButton.addActionListener(this);
        ResetButton.addActionListener(this);
        TopPanel.setBackground(Color.white);
        TopPanel.add(PlayPauseButton);
        TopPanel.add(ResetButton);
        LeftPanel.setBackground(Color.white);
        LeftPanel.setLayout(new BorderLayout(1,1));
        LeftPanel.add(TextFieldPanel, BorderLayout.NORTH);
        LeftPanel.add(PhysCalcs.Oscilloscope, BorderLayout.SOUTH);
        RightPanel.setBackground(Color.white);
        RightPanel.setLayout(new BorderLayout());       
        RightPanel.add(PhysCalcs.PhysCanvas, BorderLayout.CENTER);
        RightPanel.add(TopPanel, BorderLayout.NORTH);
        this.add(LeftPanel, BorderLayout.WEST);
        this.add(RightPanel, BorderLayout.CENTER);
        Font font 	= new Font("Arial", Font.BOLD, 12);
        setFont(font);
 }
 /**
  * This subroutine is called by the JVM after all visual components are initialized and are placed on the screen. 
  * It initializes things that cannot be initialized before they are vissible on screen. (For example when you 
  * need to know the size of a canvas for initialisation purposes.
  */ 
 public void start()
 {
 	textFMass.setText(TVector3D.Double2String(PhysCalcs.MASS, 3));
        textFLength.setText(TVector3D.Double2String(PhysCalcs.HEIGHTPENDUL, 3));
        textFalpha0.setText(TVector3D.Double2String(PhysCalcs.ALPHA0*180/Math.PI, 3));
        textFbeta0.setText(TVector3D.Double2String(PhysCalcs.BETA0*180/Math.PI, 3));
        textFK.setText(TVector3D.Double2String(PhysCalcs.K, 3));
        textFC.setText(TVector3D.Double2String(PhysCalcs.C, 3));
 	PhysCalcs.initGraphics();
 }
 /**  
  * This subroutine is called by the JVM if a button is pressed or if a value changes in a textbox. It deals with the changes
  * and resets the simulation.
  * @param A mouse click event or key event.
  */
 public void actionPerformed(ActionEvent event)
 { 	// Enter is pressed while the cursor was active in the Mass Textfield
 	if(event.getSource()==textFMass)
  	{	readTextBoxes();
  	}
  	// Enter is pressed while the cursor was active in the Length Textfield
  	else if(event.getSource()==textFLength)
  	{	readTextBoxes();
  	}
  	// Enter is pressed while the cursor was active in the alpha0 Textfield
  	else if(event.getSource()==textFalpha0)
  	{	readTextBoxes();
  	}
  	// Enter is pressed while the cursor was active in the beta0 Textfield
  	else if(event.getSource()==textFbeta0)
  	{	readTextBoxes();
  	}
  	// Enter is pressed while the cursor was active in the Spring constant Textfield
  	else if(event.getSource()==textFK)
  	{	readTextBoxes();
  	}
  	// Enter is pressed while the cursor was active in the C Textfield
  	else if(event.getSource()==textFC)
  	{	readTextBoxes();
  	}
  	// Play/Pause button is pressed
  	else if(event.getSource()==PlayPauseButton)
  	{	readTextBoxes();
  		if (timer.timerThread==null)
  		{	PhysCalcs.start();
  			timer.start();	 				
  		}
  		else
  		{	timer.switchActive();
  			PhysCalcs.switchActive();
  		}
  	}
  	// Reset button is pressed.
  	else if(event.getSource()==ResetButton)
    	{	readTextBoxes(true);
    		timer.stop();
  		PhysCalcs.stop();
  	}
  }
  /**
   * Reads out all textboxes without forcing changes
   * @see #readTextBoxes(boolean)
   */
  public void readTextBoxes()
  {	readTextBoxes(false);
  }
  /**
   * Reads all values in the textboxes, compares them to current parameters and if something has changed it resets the simulation.
   * @param forceReset if true, always force a reset.
   */
  public void readTextBoxes(boolean forceReset)
  {	boolean changed = false;
  	double TempMass = new Double(textFMass.getText()).doubleValue(); 
  	if (TempMass!=PhysCalcs.MASS)
  	{	changed = true;
  		if ((TempMass<=1000) & (TempMass>0))
  		{	PhysCalcs.MASS= TempMass;
  		}
  	}
  	double TempLength= new Double(textFLength.getText()).doubleValue();
  	if (TempLength!=PhysCalcs.HEIGHTPENDUL)
  	{	changed = true;
  		if ((TempLength<=2) & (TempLength>0))
  		{	PhysCalcs.HEIGHTPENDUL = TempLength;
  			if (PhysCalcs.C>TempLength)
  			{
  				PhysCalcs.C=TempLength*0.5;
  			}
  		}
  	}
  	double TempAlpha 	= new Double(textFalpha0.getText()).doubleValue();
  	if (TempAlpha*Math.PI/180 != PhysCalcs.ALPHA0)
  	{	changed = true;
  		if ((TempAlpha<90) & (TempAlpha>-90))
  		{	PhysCalcs.ALPHA0= TempAlpha*Math.PI/180;
  		}
   	}
  	double TempBeta		= new Double(textFbeta0.getText()).doubleValue();
  	if (TempBeta*Math.PI/180 != PhysCalcs.BETA0)
  	{	changed = true;
  		if ((TempBeta<90) & (TempBeta>-90))
  		{	PhysCalcs.BETA0	= TempBeta*Math.PI/180;
  		}
   	}
  	double TempK 		= new Double(textFK.getText()).doubleValue();
  	if (TempK!=PhysCalcs.K)
  	{	changed = true;
  		if ((TempK<=1000) & (TempK>=0))
  		{	PhysCalcs.K	= TempK; 
  		}
   	}
  	double TempC 		= new Double(textFC.getText()).doubleValue();  
  	if (TempC!=PhysCalcs.C)
  	{  	changed = true;
  		if ((TempC<=PhysCalcs.HEIGHTPENDUL) & (TempC>0))
  		{	PhysCalcs.C 	= TempC;
  		}
  	}
  	if (changed | forceReset)
  	{  	PhysCalcs.activateChanges();
  		textFalpha0.setText(TVector3D.Double2String(PhysCalcs.ALPHA0*180/Math.PI, 3));
  		textFbeta0.setText(TVector3D.Double2String(PhysCalcs.BETA0*180/Math.PI, 3));
  		textFLength.setText(TVector3D.Double2String(PhysCalcs.HEIGHTPENDUL, 3));
  		textFMass.setText(TVector3D.Double2String(PhysCalcs.MASS, 3));
  		textFC.setText(TVector3D.Double2String(PhysCalcs.C, 3));
  		textFK.setText(TVector3D.Double2String(PhysCalcs.K, 3));
  	}
  	
  }
  /**  
   * This subroutine is called by the timer object every 50 milliseconds. It orders a redraw of the physical simulation.
   */
  public void tick()
  {	PhysCalcs.PhysCanvas.redraw();
  	PhysCalcs.Oscilloscope.redraw();
  }
}  

/**
 * Class that does all the physical simulations and caries all the physical objects.
 */
class TPhysCalcs implements Runnable
{ /** First pendulum */
  public TPendulum 	Pendul1;
  /** Second pendulum */
  public TPendulum 	Pendul2;
  /** Spring between the two pendula */
  public TSpiral3D 	Spiral;
  /** Arc between LinePendul1 and the first pendulum */
  public TArc3D 	Arc3D1;
  /** Arc between LinePendul2 and the second pendulum */
  public TArc3D 	Arc3D2;
  /** Sphere at the top of the first pendulum (suspension point) */
  public TSphere3D 	Sphere1;
  /** Sphere at the top of the second pendulum (suspension point) */
  public TSphere3D 	Sphere2;
  /** Line going through the first pendulum when it is at rest. */
  public TLine3D 	LinePendul1;
  /** Line going through the second pendulum when it is at rest. */
  public TLine3D 	LinePendul2;
  /** Canvas onto which all the physical objects are drawn. */
  TPhysCanvas 		PhysCanvas;
  /** Oscilloscope canvas. */
  TOscilloscope 	Oscilloscope;
  /** Thread used for calculating and predrawing the physics of the simulation. */
  protected Thread 	timerThread;
  /** Datastream index of the first pendulum, used to identify the data for the first pendulum in the oscilloscope */
  public int 		Pendul1DataStream;
  /** Datastream index of the second pendulum, used to identify the data for the second pendulum in the oscilloscope */
  public int 		Pendul2DataStream;
  /** Time of the simulation (in milliseconds */
  public double 	t		= 0;
  /** Angular velocity of the first pendulum */
  public double 	VPendul1	= 0;
  /** Angular velocity of the second pendulum */
  public double 	VPendul2	= 0;
  /** Maximum error in the simulation */
  public double 	e_max		= 0.0005;
  /** Initial maximum stepsize (for use in adaptive stepsize) */
  public double 	h_maxinit	= 0.1;
  /** Number of seconds per frame */
  public double 	perframe	= 0.05;
  /** Initial number of steps the simulation does between two frames */
  public int 		NOTIMES		= 1;
  /** Initial mass of the pendula */
  public double 	MASS		= 1;
  /** Initial angle of the first pendulum in radians */
  public double 	ALPHA0		= 30*Math.PI/180;
  /** Initial angle of the second pendulum in radians*/
  public double 	BETA0		= 0;
  /** Initial spring constant K */
  public double 	K		= 4;
  /** Initial Height of the pendula */
  public double 	HEIGHTPENDUL	= 1;
  /** Width of the pendula */
  public double 	WIDTHPENDUL	= 0.05;
  /** Depth of the pendula */
  public double 	DEPTHPENDUL	= 0.03;
  /** Distance between center of mass and suspension point of the pendula */
  public double 	CM2AXIS		= HEIGHTPENDUL*0.5;
  /** Gravitational constant */
  public double 	G		= 9.81;
  /** Distance between suspension point of the pendulum and suspension point of the spring ON the pendulum */
  public double 	C		= CM2AXIS;
  /** Stepsize */
  private double 	h;
  /** Activity flag */
  public boolean	Active		= false;
  /** 
   * The constructor of the class. It initializes all the physical components
   */
  TPhysCalcs()
  {	PhysCanvas 	= new TPhysCanvas(-HEIGHTPENDUL,-HEIGHTPENDUL*2,HEIGHTPENDUL,HEIGHTPENDUL*0.1);
  	Oscilloscope 	= new TOscilloscope(300,250, 10,-ALPHA0*360/Math.PI,ALPHA0*360/Math.PI,Color.black);
  	Sphere1		= new TSphere3D(-HEIGHTPENDUL*0.25,0,DEPTHPENDUL, WIDTHPENDUL, Color.black);
  	Sphere2		= new TSphere3D(HEIGHTPENDUL*0.25,0,DEPTHPENDUL, WIDTHPENDUL, Color.black);
        Arc3D1		= new TArc3D(new TVector3D(-HEIGHTPENDUL*0.25,0,0), HEIGHTPENDUL*1.2, 1.5*Math.PI, ALPHA0,new String("\u03B1") ,  Color.darkGray);
        Arc3D2		= new TArc3D(new TVector3D(HEIGHTPENDUL*0.25,0,0), HEIGHTPENDUL*1.2, 1.5*Math.PI, BETA0,new String("\u03B2") ,  Color.darkGray);
        Pendul1 	= new TPendulum(-WIDTHPENDUL*0.5-HEIGHTPENDUL*0.25,0,-DEPTHPENDUL*0.5,MASS,HEIGHTPENDUL,WIDTHPENDUL,DEPTHPENDUL, Color.blue);
     	Pendul2 	= new TPendulum(-WIDTHPENDUL*0.5+HEIGHTPENDUL*0.25,0,-DEPTHPENDUL*0.5, MASS,HEIGHTPENDUL,WIDTHPENDUL,DEPTHPENDUL,Color.red);
       	LinePendul1	= new TLine3D(-HEIGHTPENDUL*0.25,-2*HEIGHTPENDUL, -DEPTHPENDUL*0.5, -HEIGHTPENDUL*0.25,HEIGHTPENDUL*1,-DEPTHPENDUL*0.5, Color.darkGray);
     	LinePendul2	= new TLine3D(HEIGHTPENDUL*0.25,-2*HEIGHTPENDUL,-DEPTHPENDUL*0.5, HEIGHTPENDUL*0.25,HEIGHTPENDUL*1,-DEPTHPENDUL*0.5, Color.darkGray);
        Pendul1DataStream= Oscilloscope.newDataStream(Color.blue);
     	Pendul2DataStream= Oscilloscope.newDataStream(Color.red);
     	// Set the fixed axis for rotation of both pendula.
     	Pendul1.setFixedAxis(new TVector3D(-HEIGHTPENDUL*0.25, 0,0), TVector3D.StdZaxis(),TPendulum.GLOBAL);
     	Pendul2.setFixedAxis(new TVector3D(HEIGHTPENDUL*0.25,0,0), TVector3D.StdZaxis(),TPendulum.GLOBAL);
     	// Rotate both pendula 180 degrees, so they hang below their suspension point
     	Pendul1.rotAroundFixedAxis(Math.PI);
     	Pendul2.rotAroundFixedAxis(Math.PI);
     	// Set points on the pendula to track for determining the angle
     	Pendul1.setPickPoint(new TVector3D(-HEIGHTPENDUL*0.25,-C,0),TPendulum.GLOBAL,0);
     	Pendul2.setPickPoint(new TVector3D(HEIGHTPENDUL*0.25,-C,0),TPendulum.GLOBAL,0);
     	// Rotate the pendula with the initial angles
     	Pendul1.rotAroundFixedAxis(ALPHA0);
     	Pendul2.rotAroundFixedAxis(BETA0);
     	// Now that the pendula are in their initial positions, the spiral can be hung in place.
     	Spiral		= new TSpiral3D(Pendul1.getPickPoint(0), Pendul2.getPickPoint(0), WIDTHPENDUL*0.5, 10, 8, Color.black);
        // Append all these objects to the objectlist for rendering.
        PhysCanvas.Append(Pendul1);
     	PhysCanvas.Append(Pendul2);
     	PhysCanvas.Append(LinePendul1);
     	PhysCanvas.Append(LinePendul2);
     	PhysCanvas.Append(Arc3D1);
     	PhysCanvas.Append(Arc3D2);
     	PhysCanvas.Append(Sphere1);
     	PhysCanvas.Append(Sphere2);
        PhysCanvas.Append(Spiral);
        Oscilloscope.setBackground(Color.white);
  	PhysCanvas.setBackground(Color.white);
	// Use adaptive stepsize on a 1 period simulation of a single pendulum. The smallest stepsize is used throughout the experiment
  	estimateStepsize();
  	// Set time and both velocities zero.
      	t	=0;
      	VPendul1=0;
      	VPendul2=0;   
   }
  /** 
   * Java's arctangens is only defined in a certain range of points, this is a work around for angles > 0.5 PI
   * @param _alpha The angle in radians for which the arctangens should be calculated
   * @returns the arctangens of _alpha.
   */
  public double atan(double _alpha)
  {  	double result = Math.atan(_alpha);
  	if ((result>Math.PI/2) | (result<-Math.PI/2))
  	{	result*=-1;
  	}
  	return result;
  }
  /**
   * After a parameter is changed this routine is called. It resets all physical elements.  	
   */	
  public void activateChanges()
  {  	//Record the state of the simulation prior to changing.
  	boolean oldActive	= Active;
	Active			= false;
  	// set time =0 and all velocities zero
  	t			= 0;
      	VPendul1		= 0;
      	VPendul2		= 0;
      	CM2AXIS			= HEIGHTPENDUL*0.5;
        // Set the height, width, depth and position of both pendula
  	Pendul1.setValues(-WIDTHPENDUL*0.5-HEIGHTPENDUL*0.25,0,-DEPTHPENDUL*0.5,MASS,HEIGHTPENDUL,WIDTHPENDUL,DEPTHPENDUL,Color.blue);
  	Pendul2.setValues(-WIDTHPENDUL*0.5+HEIGHTPENDUL*0.25,0,-DEPTHPENDUL*0.5, MASS,HEIGHTPENDUL,WIDTHPENDUL,DEPTHPENDUL,Color.red);
  	// Set the fixed axis of both pendula.
  	Pendul1.setFixedAxis(new TVector3D(-HEIGHTPENDUL*0.25,0,0), TVector3D.StdZaxis(),TPendulum.GLOBAL);
  	Pendul2.setFixedAxis(new TVector3D(HEIGHTPENDUL*0.25,0,0), TVector3D.StdZaxis(),TPendulum.GLOBAL);
  	// Again rotate with angle PI to bring them in there rest position
  	Pendul1.rotAroundFixedAxis(Math.PI);
  	Pendul2.rotAroundFixedAxis(Math.PI);
  	// Reset the values of both vertical lines
  	LinePendul1.Line3D(-HEIGHTPENDUL*0.25,-2*HEIGHTPENDUL,-DEPTHPENDUL*0.5, -HEIGHTPENDUL*0.25,HEIGHTPENDUL*1,-DEPTHPENDUL*0.5);
     	LinePendul2.Line3D(HEIGHTPENDUL*0.25,-2*HEIGHTPENDUL,-DEPTHPENDUL*0.5, HEIGHTPENDUL*0.25,HEIGHTPENDUL*1,-DEPTHPENDUL*0.5);
     	// Reset the values of both suspension points
     	Sphere1.Sphere3D(-HEIGHTPENDUL*0.25,0,DEPTHPENDUL, WIDTHPENDUL);
  	Sphere2.Sphere3D(HEIGHTPENDUL*0.25,0,DEPTHPENDUL, WIDTHPENDUL);
  	// Reset the values of both Arcs
  	Arc3D1.Arc3D(new TVector3D(-HEIGHTPENDUL*0.25,0,0), HEIGHTPENDUL*1.2, 1.5*Math.PI, ALPHA0,new String("\u03B1"));
        Arc3D2.Arc3D(new TVector3D(HEIGHTPENDUL*0.25,0,0), HEIGHTPENDUL*1.2, 1.5*Math.PI, BETA0,new String("\u03B2"));
        // Reset the values of both pickpoints.
  	Pendul1.setPickPoint(new TVector3D(-HEIGHTPENDUL*0.25,-C,0),TPendulum.GLOBAL,0);
  	Pendul2.setPickPoint(new TVector3D(HEIGHTPENDUL*0.25,-C,0),TPendulum.GLOBAL,0);
  	// Rotate into the initial position.
  	Pendul1.rotAroundFixedAxis(ALPHA0);
 	Pendul2.rotAroundFixedAxis(BETA0);
 	// Set the spirals begin and endpoint to the correct values
  	Spiral.setBeginNEndPoint(Pendul1.getPickPoint(0),Pendul2.getPickPoint(0));
  	// Estimate the stepsize again
 	estimateStepsize();
 	// Clear the 2 datastreams in the oscilloscoop.
      	Oscilloscope.clearDataStream(Pendul1DataStream);
      	Oscilloscope.clearDataStream(Pendul2DataStream);
      	// Set the Oscilloscopes maximum Y values to half of the maximum of Alpha or Beta
      	double maxY 		= Math.abs(ALPHA0);
      	if (Math.abs(BETA0)>maxY)
      	{	maxY		= Math.abs(BETA0);
	}
	maxY 			*= 180/Math.PI;
	maxY 			= Math.round(maxY*0.2)*10;
	Oscilloscope.setBoundaries(10, -maxY, maxY);
	// Initialize the graphics. Redraw buffer.
      	initGraphics();
      	// Reset the state of the simulation
        Active 			= oldActive;
  }
  /**
   * When called this routine switches the state of the simulation. If it runs, it gets paused, if it is paused it will start runnign again.
   */  	
  public void switchActive()
  {	if (timerThread==null) 	
  	{	start();
  	}
	else
	{	stop();
	}
  }
  /**
   * Call the initGraphics of both canvases. These initialize the canvas. Initialize the double buffering, a standard technique
   * used in computer animations to get a flicker free image.
   */	
  public void initGraphics()
  {  	 PhysCanvas.initGraphics();
  	 Oscilloscope.initGraphics();
  }
  /**	
   * start the simulation by setting both canvases to active, and opening a new thread that calls the run method.
   */
  public void start()
  {  	PhysCanvas.setActive();
  	Oscilloscope.setActive();
  	this.setActive();
  	timerThread = new Thread(this);
    	timerThread.start();
  }
  /**	
   * stop the simulation by setting both canvases to inactive, and shutting down the thread.
   */
  public void stop()
  { 	PhysCanvas.setInactive();
  	Oscilloscope.setInactive();
  	this.setInactive();
  	timerThread = null; 
  }
  /**
   * This routine is called when a thread is started for this class. It runs until the thread is killed
   */
  synchronized public void run()
  {  	
  	Thread thisThread = Thread.currentThread();
  	while (timerThread==thisThread) // make sure that the loop does not go on while the thread is already killed
  	{	
 		if(Active) // Check again if the thread is still active.
 		{	// Draw the canvasses to the buffer. And check to see if actual drawing was done
 			boolean PhysDrawn 	= PhysCanvas.bufferDraw();
 			boolean OscDrawn 	= Oscilloscope.bufferDraw();
 			if (!(PhysDrawn&OscDrawn))
 			{	// if the buffer is full, sleep for 50 milliseconds.
 				try
 				{	timerThread.sleep(50);
 				}
 				catch (Exception e)
 				{ System.out.println(e);}
 			}
 			else if(Active)
 			{	// Get alpha and beta out of the position of the pickpoint. Set dalpha and dbeta to zero, these two will determine 
 				// how much the pendula should be rotated relative to their current rotation.
 				TVector3D pickpoint1 	= new TVector3D(Pendul1.getPickPoint(0));
  				TVector3D pickpoint2 	= new TVector3D(Pendul2.getPickPoint(0));
  				
        			double alpha		= atan(-(pickpoint1.x+HEIGHTPENDUL*0.25)/pickpoint1.y) ;
 				double dalpha		= 0;
 				double beta		= atan(-(pickpoint2.x-HEIGHTPENDUL*0.25)/pickpoint2.y);
 				double dbeta		= 0;
 				double alphaOld		= alpha;
 				double betaOld		= beta;
 				double temp		= 0;
 				int inc			= 0;
 				// Set the time to the number of simulations to be done in this loop times the stepsize
 				t			+= NOTIMES*h;
 				for (int c1=0; c1<NOTIMES; c1++)
 				{	// Do the integration
 					temp 		= VPendul1*h+Accel(alphaOld,betaOld)*h*h*0.5;
 					alpha		+= temp;
 					dalpha		+= temp;
 					temp		= VPendul2*h+Accel(betaOld,alphaOld)*h*h*0.5;
 					beta		+= temp;
 					dbeta		+= temp;
   					VPendul1	+= (Accel(alphaOld,betaOld) + Accel(alpha, beta))  *h*0.5;
   					VPendul2	+= (Accel(betaOld ,alphaOld)+ Accel(beta, alpha))  *h*0.5;
   					alphaOld 	= alpha;
   					betaOld		= beta;
   				}
   				// Rotate the pendula to there new position
   				Pendul1.rotAroundFixedAxis(dalpha);
 				Pendul2.rotAroundFixedAxis(dbeta);
 				// Set the spiral to the right values.
 				Spiral.setBeginNEndPoint(Pendul1.getPickPoint(0),Pendul2.getPickPoint(0));
 				// Add this value of alpha and beta to the oscilloscope
 				Oscilloscope.addMeas(NOTIMES*h,alpha*180/Math.PI,0,Pendul1DataStream);	
 				Oscilloscope.addMeas(NOTIMES*h,beta *180/Math.PI,0,Pendul2DataStream);
 				// Change the values for the arcs.
 				Arc3D1.changeAngle(alpha);
 				Arc3D2.changeAngle(beta);
 			}
 		}
 		else
 		{	try
 			{	thisThread.sleep(100);
 			}
 			catch (Exception e)
 			{}
 		}
 	}
  }  
  /**
   * Angular acceleration off the pendula. 
   * @param _a the deviation from the equilibrium position of the pendulum for which the acceleration should be calculated
   * @param _b the deviation from the equilibrium position of the other pendulum
   * @returns the angular acceleration of this pendulum
   */
  public double Accel(double _a, double _b)
  {	return -(MASS*G*CM2AXIS*Math.sin(_a) + K*C*C*(Math.sin(_a)-Math.sin(_b)))/Inertia();
  }
  /**
   * Angular acceleration of a single pendulum
   * @param _a the deviation from the equilibrium position of the pendulum
   * @returns the angular acceleration of the pendulum.
   */
  public double Accel(double _a)
  {	return -(MASS*G*CM2AXIS*Math.sin(_a))/Inertia();
  } 
  /**
   * Set Active true
   */
  public void setActive()
  {  	Active=true;
  }
  /**
   *Set Active false
   */  
  public void setInactive()
  {  	Active=false;
  }
  /**
   * The Inertia of the pendula
   * @retuns the inertia of the pendula
   */
  public double Inertia()
  {    	return MASS*(HEIGHTPENDUL*HEIGHTPENDUL+WIDTHPENDUL*WIDTHPENDUL)/12+MASS*CM2AXIS*CM2AXIS;
  }
  /**
   * Use adaptive stepsize velocity verlet on 1 period of simulation of a single pendulum. 
   * The smallest stepsize used is the stepsize that is going
   * to be used by the rest of the simulation.
   */ 
  public void estimateStepsize()
  {   	
  	// Get alpha for the first pendulum and set dalpha 0.
  	TVector3D pickpoint1 	= new TVector3D(Pendul1.getPickPoint(0));
 	double alpha		= atan((-pickpoint1.x+HEIGHTPENDUL*0.25)/pickpoint1.y) ;
 	double dalpha		= 0;
  	// Set all parameters for the adaptive stepsize.
  	boolean ready		= false;
  	double h_max		= h_maxinit;
 	h			= h_max;
 	double SmallestStep	= 1;
 	int inc			= 0;
 	double v		= 0;
 	double alpha1;
 	double alpha2;
 	double alpha1Old;
 	double alpha2Old;
 	double v1	= v;
 	double v2	= v;
 	double period 	= 2*Math.PI * Math.sqrt(MASS*G*CM2AXIS/Inertia());
 	while(dalpha<period) // Run the simulation till the absolute cummulative of dalpha is larger than 2PI
 	{	ready		= false;
 		while(!ready & inc<100) // Run this part until the stepsize is right, or until this part is looped 100 times.
 		{	alpha1		= alpha;
 			alpha2		= alpha;
 			alpha1Old	= alpha;
 			alpha2Old	= alpha;
 			v1		= v;
 			v2		= v;
 			// Calculate alpha 1 by doing 1 step of size h and calculate alpha 2 by doing 2 steps of size h/2
   			alpha1		+= v1*h +Accel(alpha1Old)*h*h*0.5 ;
   			v1		+= (Accel(alpha1)+Accel(alpha1Old))*h*0.5;
 			alpha2		+= v2*h*0.5 +Accel(alpha2Old)*h*h*0.25;;
 			v2		+= (Accel(alpha2)+Accel(alpha2Old))*h*0.5;
 			alpha2Old	= alpha2;
 			alpha2		+= v2*h*0.5 +Accel(alpha2Old)*h*h*0.25;;
 			v2		+= (Accel(alpha2)+Accel(alpha2Old))*h*0.5;
 			// the estimate of the error is the differense between the 2 alpha's. 
 			double e_h	= (8.0/7.0)*Math.abs(alpha2-alpha1);
 			// Recalculate h_max out of the maximum error and the obtained error.
 			h_max		= h*Math.pow(Math.abs(e_max/e_h),0.25);
 			if (h_max<(h/2))
 			{	// if the error was too large redo the simulation
 				h = 1.8*h_max;
 				inc++;
 				ready	= false;
 			}
 			else
 			{	// if the error is right step out of the loop and do the folowing time step. 
  				// Keep track of the smallest step done in the entire simulation
 				dalpha	+= Math.abs(alpha2-alpha);
 				alpha	= alpha2;
 				v	= v2;
 				ready	= true;
 				inc	= 0;
 				if (h_max<SmallestStep) 
 				{	SmallestStep=h_max;
 				}
 			}
 		}
 	}
 	// The stepsize used in the entire double pendulum simulation is the smallest step in the adaptive stepsize Euler simulation
 	h	= SmallestStep;
 	// Calculate the number of steps to be done in 1 draw of the simulation.
 	NOTIMES	= (int)(Math.floor(perframe/h)+1);
 	// Set the velocity to zero again.
 	v	= 0;
 	
  }  
} 